Simple PWM controller
=====================

<https://bues.ch/cgit/simplepwm.git/>

Simplepwm is a small PWM brightness controller for LED lamps.

Microcontroller
===============

Simplepwm supports the following microcontrollers. Not all features are
supported on all microcontrollers. See the table for details.

+---------+---------+---------+---------+---------+---------+---------+
| ..      | ATMe    | ATMeg   | AT      | AT      | AT      | AT      |
|         | ga-328P | a-88(P) | Tiny-85 | Tiny-45 | Tiny-25 | Tiny-13 |
+=========+=========+=========+=========+=========+=========+=========+
| Number  | > 3     | > 3     | > 1     | > 1     | > 1     | > 1     |
| of      |         |         |         |         |         |         |
| analog  |         |         |         |         |         |         |
| pot.    |         |         |         |         |         |         |
| inputs  |         |         |         |         |         |         |
+---------+---------+---------+---------+---------+---------+---------+
| Number  | > 3 x x | > 3     | > 1 x x | > 1 x x | > 1     | > 1     |
| of PWM  | > x x x | >       |         |         |         |         |
| outputs | > x x   | > x x x |         |         |         |         |
| Power   |         | > x x   |         |         |         |         |
| saving  |         |         |         |         |         |         |
| Battery |         |         |         |         |         |         |
| voltage |         |         |         |         |         |         |
| mon     |         |         |         |         |         |         |
| itoring |         |         |         |         |         |         |
| HSL     |         |         |         |         |         |         |
| color   |         |         |         |         |         |         |
| model   |         |         |         |         |         |         |
| Remote  |         |         |         |         |         |         |
| control |         |         |         |         |         |         |
| via     |         |         |         |         |         |         |
| UART    |         |         |         |         |         |         |
| S       |         |         |         |         |         |         |
| ettings |         |         |         |         |         |         |
| in      |         |         |         |         |         |         |
| EEPROM  |         |         |         |         |         |         |
| De      |         |         |         |         |         |         |
| bugging |         |         |         |         |         |         |
| via     |         |         |         |         |         |         |
| UART    |         |         |         |         |         |         |
| F       |         |         |         |         |         |         |
| irmware |         |         |         |         |         |         |
| update  |         |         |         |         |         |         |
| via     |         |         |         |         |         |         |
| UART    |         |         |         |         |         |         |
+---------+---------+---------+---------+---------+---------+---------+

Hardware schematics
===================

Schematics for ATMega based setups:
[schematics-atmega/simplepwm-atmega.pdf](schematics-atmega/simplepwm-atmega.pdf)

Schematics for ATTiny based setups:
[schematics-attiny/simplepwm-attiny.pdf](schematics-attiny/simplepwm-attiny.pdf)

Prebuilt firmware images
========================

The release archives of simplepwm contain prebuilt [.hex]{.title-ref}
files for all supported microcontrollers in the [hex]{.title-ref}
directory. These hex files can be flashed directly to the
microcontroller with a any tool of your choice (e.g. Atmel Studio or
avrdude).

The corresponding fuse settings can be found in the file
[fuses.txt]{.title-ref} in the same directory.

Building the firmware
=====================

The firmware build needs the following tool chain:

-   Unix-like operating system
-   GNU make
-   AVR GCC
-   AVR Binutils
-   avrdude

Run the following commands to build the firmware:

``` {.sh}
cd firmware
make DEV=t85
```

Please specify the target microcontroller using the DEV variable as
shown above.

Valid values are:

  Microcontroller   make option
  ----------------- -------------------------
  ATTiny 13         [DEV=t13]{.title-ref}
  ATTiny 25         [DEV=t25]{.title-ref}
  ATTiny 45         [DEV=t45]{.title-ref}
  ATTiny 85         [DEV=t85]{.title-ref}
  ATMega 88         [DEV=m88]{.title-ref}
  ATMega 88P        [DEV=m88p]{.title-ref}
  ATMega 328P       [DEV=m328p]{.title-ref}

Additional build options that can be passed to \`make\`:

  make option       values   default          description
  ----------------- -------- ---------------- ------------------------------------------------------------------
  CONF\_PWMLIM      1-100    100              Limit the maximum PWM duty cycle to this percentage.
  CONF\_PWMINVERT   0-1      0                Invert the PWM output signal(s).
  CONF\_LOWSPFACT   float    1.0              Correction factor for lower range PWM setpoint.
  CONF\_ADCINVERT   0-1      0                Invert the ADC input signal(s).
  CONF\_ADCHSL      0-1      depends on DEV   Interpret the ADC input signals as HSL setpoints instead of RGB.
  FEAT\_HIGHRES     0-1      depends on DEV   Higher resolution in very low PWM setpoint range.
  FEAT\_BATTERY     0-1      depends on DEV   Battery voltage monitoring
  FEAT\_POWERSAVE   0-1      depends on DEV   Power saving / deep sleep
  FEAT\_REMOTE      0-1      depends on DEV   Remote control via UART
  FEAT\_EEPROM      0-1      depends on DEV   Store settings to EEPROM
  FEAT\_BOOT        0-1      depends on DEV   Enable bootloader and firmware updating via UART.

Flashing the firmware via serial bootloader after build
=======================================================

Run the following commands to flash the firmware via serial bootloader
to the target microcontroller.

This only works, if there\'s already a previous version of SimplePWM
running on the chip. If the chip is blank, refer to ISP flashing below.

``` {.sh}
cd firmware
./flash-usb.sh /dev/ttyUSB0
```

Flashing the firmware via ISP adapter after build
=================================================

Run the following commands to flash the firmware via ISP (In System
Programmer hardware adapter) to the target microcontroller:

``` {.sh}
cd firmware
make DEV=t85 print_fuses
make DEV=t85 write_fuses
make DEV=t85 write_mem
```

Or alternatively use any other of the available AVR flashing tools to
program the .hex file and fuses.

License
=======

Copyright (c) 2018-2021 Michael Buesch \<<m@bues.ch>\>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
